<?php
/**
 * @version     0.0.3
 * @package     com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gasecminutes\Administrator\Model;
// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\MVC\Model\ListModel;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Helper\TagsHelper;
use \Joomla\Database\ParameterType;
use \Joomla\Utilities\ArrayHelper;

/**
 * Methods supporting a list of records.
 * @since  1.6
 */
class SecminsModel extends ListModel
{
    
        
/**
	* Constructor.
	* @param   array  $config  An optional associative array of configuration settings.
	* @see        JController
	* @since      1.6
	*/
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'ordering', 'a.ordering',
				'state', 'a.state',
				'created_by', 'a.created_by',
				'modified_by', 'a.modified_by',
				'created_date', 'a.created_date',
				'modified_date', 'a.modified_date',
				'meet_id', 'a.meet_id', 'meet_id_name',
				'cat_id', 'a.cat_id', 'cat_id_name',
				'm.cat_id', 'meet_cat_id_name',
				'comment', 'a.comment',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 * @return void
	 * @throws Exception
	 */
	protected function populateState($ordering = null, $direction = null)
	{
        // List state information.
        parent::populateState("m.start_date", "DESC");

        $context = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
        $this->setState('filter.search', $context);

        $state = $this->getUserStateFromRequest($this->context.'.filter.state', 'filter_state');
        $this->setState('filter.state', $state);

        $catid = $this->getUserStateFromRequest($this->context.'.filter.cat_id', 'filter_cat_id');
        $this->setState('filter.cat_id', $catid);

	}

	/**
	 * Method to get a store id based on model configuration state.
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 * @param   string  $id  A prefix for the store id.
	 * @return   string A store id.
	 * @since    1.6
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');
		$id .= ':' . $this->getState('filter.state');

        return parent::getStoreId($id);
                
	}

	/**
	 * Build an SQL query to load the list data.
	 * @return   DatabaseQuery
	 * @since    1.6
	 */
	protected function getListQuery()
	{
        // Process select filters.
        $status    = $this->getState('filter.state', '');
        $catid = $this->getState('filter.cat_id', '');
        $search = $this->getState('filter.search', '');

		// Create a new query object.
        $db    = $this->getDatabase();
        $query = $db->getQuery(true)
            ->select('a.*')
            ->select(
                [
                    $db->quoteName('uChecked.name', 'uEditor'),
                    $db->quoteName('uCreated.name', 'uCreator'),
                    $db->quoteName('uModified.name', 'uModifier'),
                    $db->quoteName('catId.title', 'cat_id_name'),
                    $db->quoteName('mcatId.title', 'meet_cat_id_name'),
                    $db->quoteName('m.title', 'meet_id_name'),
                    $db->quoteName('m.start_date', 'start_date'),
                ]
            )
            ->from($db->quoteName('#__gasecminutes_secmins', 'a'))
            ->join('LEFT', $db->quoteName('#__users', 'uChecked'), $db->quoteName('uChecked.id') . ' = ' . $db->quoteName('a.checked_out'))
            ->join('LEFT', $db->quoteName('#__users', 'uCreated'), $db->quoteName('uCreated.id') . ' = ' . $db->quoteName('a.created_by'))
            ->join('LEFT', $db->quoteName('#__users', 'uModified'), $db->quoteName('uModified.id') . ' = ' . $db->quoteName('a.modified_by'))
            ->join('LEFT', $db->quoteName('#__categories', 'catId'), $db->quoteName('catId.id') . ' = ' . $db->quoteName('a.cat_id'))
            ->join('LEFT', $db->quoteName('#__gasecminutes_meetings', 'm'), $db->quoteName('m.id') . ' = ' . $db->quoteName('a.meet_id'))
            ->join('LEFT', $db->quoteName('#__categories', 'mcatId'), $db->quoteName('mcatId.id') . ' = ' . $db->quoteName('m.cat_id'));

		// Filter by category
		if ($catid) {
            $catid = (int) $catid;
            $query->where($db->quoteName('a.cat_id') . ' = :catid')
                ->bind(':catid', $catid, ParameterType::INTEGER);
        }

		// Filter by state
		if (is_numeric($status)) {
            $status = (int) $status;
            $query->where($db->quoteName('a.state') . ' = :status')
                ->bind(':status', $status, ParameterType::INTEGER);
		} elseif ($status === '*') {
			// show all and don't filter on status
		} else {
			$query->where($db->quoteName('a.state') . ' IN (0, 1) ');
        }

		// Filter by search in field
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$query->where('id = ' . (int) substr($search, 3));
            } else {
                $search = '%' . str_replace(' ', '%', trim($search)) . '%';
                $query->where($db->quoteName('catId.title') . ' LIKE :search || '.$db->quoteName('mcatId.title') . ' LIKE :search')
                    ->bind(':search', $search);
            }
		}
                
		// Add the list ordering clause.
		$orderCol  = $this->state->get('list.ordering', 'm.start_date');
		$orderDirn = $this->state->get('list.direction', 'DESC');

		if ($orderCol && $orderDirn) {
			$query->order($db->escape($orderCol . ' ' . $orderDirn));
		}

		return $query;
	}

	/**
	 * Get an array of data items
	 * @return mixed Array of data items on success, false on failure.
	 */
	public function getItems()
	{
		$items = parent::getItems();
                
		return $items;
	}
}
